//
// dsPIC30 PWM DAC for JUMA-TRX2
// Juha Niinikoski, OH2NLT 23.11.2006
//
// Microchip C30 compiler conversion
//
//
// init TMR3 & OC1, OC2, OC3 and OC4 for PWM operation
//
// OC1 output is for SCAF filter clock
// OC1 is connected to TMR2
//
// OC2 output is for CW sidetone out
// OC3 output is for LCD back light control
// OC4 output is for LCD contrast
// OC2...OC4 are connected to TMR3
// TMR3 is set for side 700Hz tone
// Period = 700 Hz
// control range = 0 - 10714
//

#define STONE_PERIOD 10714
#define SCAF_PERIOD 150

#include "juma-trx2.h"


void init_pwm_dac(unsigned int pwm1, unsigned int pwm2, unsigned int pwm3, unsigned int pwm4)
	{
	OC1CON = 0;			// first turn off the module if it was on
	OC2CON = 0;
	OC3CON = 0;
	OC4CON = 0;

	OC1RS = pwm1;		// set OC1 initial duty cycle
	OC1R = pwm1;
	OC1CON = 0x0006;	// use TMR2, OC1 = PWM mode

	OC2RS = pwm2;		// set OC2 initial duty cycle
	OC2R = pwm2;
	OC2CON = 0x000E;	// use TMR3, OC2 = PWM mode

	OC3RS = pwm3;		// set OC3 initial duty cycle
	OC3R = pwm3;
	OC3CON = 0x000E;	// use TMR3, OC3 = PWM mode

	OC4RS = pwm4;		// set OC4 initial duty cycle
	OC4R = pwm4;
	OC4CON = 0x000E;	// use TMR3, OC4 = PWM mode

	PR3 = STONE_PERIOD;			// set PWM cycle, TMR3 period register, 700Hz
	T3CONbits.TON = 1;			// start TMR3

	PR2 = SCAF_PERIOD;			// SCAF clock frequency, 50kHz
	T2CONbits.TON = 1;			// start TMR2	
	}

void set_pwm1_dac(unsigned int pwm)
	{
	OC1RS = pwm;
	}

void set_pwm2_dac(unsigned int pwm)
	{
	OC2RS = pwm;
	}

void set_pwm3_dac(unsigned int pwm)
	{
	OC3RS = pwm;
	}

void set_pwm4_dac(unsigned int pwm)
	{
	OC4RS = pwm;
	}
