//
// JUMA-TRX2 external serial keyboard for Voice Memory control
//
// Juha Niinikoski, OH2NLT 12.03.2007
//
// User interface "Type 2" added 14.03.2007
// Voice memory option installed flag added 17.03.2007

#include <stdio.h>
#include "juma-trx2.h"				// board definitions
#include "lcd-trx2.h"				// LCD specific definitions

// EEPROM structures
#include "trx2_eeprom.h"			// get EEPROM structure definitions

extern union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval)/2];
	}eeprom;

// system calibration values

extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval)/2];
	}cal;


// external references
// external functions

extern void putch(char);						// uart I/O
extern int kbhit(void);
extern unsigned char getch(void);
extern unsigned char getche(void);
extern unsigned long getlong(void);
extern void bin2ascii(unsigned long);
extern void putlong( unsigned long); 

extern void lcd_putst(register const char *);	// LCD I/O
extern void clear_lcd(void);
extern void set_cur_lcd(unsigned char);
extern void disp_freq(long);
extern void disp_rit(int);
extern void disp_meter(int, int);

extern void exchg_data_spi1(unsigned int);

extern void disp_bar_meter(void);				// measure, calculate & display PWR or S meter

// ISD1700 test
extern void init_spi2(void);
extern unsigned int tx_rx_16_spi2(unsigned int dta);
extern unsigned int tx_rx_32_spi2(unsigned int cmd1, unsigned int dta2 );
extern unsigned int tx_rx_56_spi2(unsigned int cmd, unsigned int dta1, unsigned int dta2 );
extern void set_isd_addresses(int idx);
extern void play_mem(int idx, int mode);
extern void rec_mem(int idx, int ch);

// sound effect
extern void beep(int, int);							// generate beep, period, length

// scaf clock control
extern void stop_scaf_clock(void);					// stop scaf clocking
extern void start_scaf_clock(void);					// start scaf clocking

// external data

extern int service_mode;				// service mode selected flag
extern unsigned int long_push;			// long push timer for buttons
extern int sq_timer;					// squelch timer

// module variables
static int voice_mem;							// selected voice memory bank, 0-9
extern int isd_1700_installed;			// voice memory board found


#if 0
// Type 1 interface
// Select memory first and then issue command

void serial_ext_kb(void)
	{
	int x;
	unsigned char c;

		if(kbhit())
			{
			c = getch();

			if(isd_1700_installed == 0)
				c = 0;					// void input if module not installed

			switch(c)
				{

// ISD1700 commands

// select memory bank
// 0 = long memory for recording from RX
// others are short message memories
				case '0' :case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
					voice_mem = c-'0';
					printf("\n\rMem Select = %i", voice_mem);
					set_isd_addresses(voice_mem);
					break;

// play to speaker
				case 'P':
					printf("\n\rISD1700 Play mem");
					play_mem(voice_mem, 0);
					break;

// TX on the air
				case 'T':
					printf("\n\rISD1700 TX mem");
					ISD_PLAY_BAND = 1;					// connect ISD1700 to mic ch
					PTT_OUT = 1;						// TX on
					play_mem(voice_mem, 1);				// Play message, stop at EOM oe end address
					PTT_OUT = 0;						// TX off
					ISD_PLAY_BAND = 0;					// disconnect ISD1700 from MIC ch
					break;

// Record from RX audio
				case 'R':
					printf("\n\rISD1700 Rec RX");
					rec_mem(voice_mem, 1);
					break;

// Record from microphone
				case 'M':
					printf("\n\rISD1700 Rec Mic");
					rec_mem(voice_mem, 0);
					break;

// service & test commands

				case 'S':
					printf("\n\rStop current operation");
					tx_rx_16_spi2(0x0002 | 0x0010);
					break;

// erase whole chip
				case 'E':
					printf("\n\rISD1700 Erase all");
					tx_rx_16_spi2(0x0043 | 0x0010);
					break;

// option not installed
				case 0:
					printf("\n\rISD1700 not installed");
					break;

				default:
					printf("?");

				}	// case
			}		// if
	}
#endif

// Type 2 interface, this is in use
// Select function first and then start with memory number select

static unsigned char cmd;												// stored command

void serial_ext_kb(void)
	{
	int x;
	unsigned char c;

		if(kbhit())
			{
			c = getch();

			if(isd_1700_installed == 0)
				c = 0;					// void input if module not installed

			switch(c)
				{
// store valid commands

				case 'P': case 'T': case 'R': case 'M':
					cmd = c;
					beep(HZ587_31, cal.calval.beep_len);				// short beep for push noticed
					printf("\n\rVoice Mem cmd = ");
					putch(c);											// echo command
					break;

// service & test commands, excecute immediatelly

				case 'S':
					printf("\n\rVoice Mem Stop operation");
					tx_rx_16_spi2(0x0002 | 0x0010);
					beep(HZ587_31, cal.calval.beep_len);				// short beep for push noticed
					break;

				case 'E':
					printf("\n\rVoice Memory erase all");
					beep(HZ587_31, cal.calval.beep_len*10);				// long beep for push noticed
					tx_rx_16_spi2(0x0043 | 0x0010);
					cmd = 0;						// void command
					break;

// select memory bank
// 0 = long memory for recording from RX
// others are short message memories
				case '0' :case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
					voice_mem = c-'0';
					printf("\n\rMem = %i", voice_mem);
					set_isd_addresses(voice_mem);

// start stored command

				switch(cmd)
					{
// ISD1700 commands

// play to speaker
					case 'P':
						printf("\n\rVM, play memory");
						play_mem(voice_mem, 0);
						break;

// TX on the air
					case 'T':
						printf("\n\rVM, TX");
						ISD_PLAY_BAND = 1;					// connect ISD1700 to mic ch
						PTT_OUT = 1;						// TX on
						play_mem(voice_mem, 1);				// Play message, stop at EOM or end address
						PTT_OUT = 0;						// TX off
						ISD_PLAY_BAND = 0;					// disconnect ISD1700 from MIC ch
						break;

// Record from RX audio
					case 'R':
						printf("\n\rVM RX");
						rec_mem(voice_mem, 1);
						break;

// Record from microphone
					case 'M':
						printf("\n\rVM, Record from Mic");
						rec_mem(voice_mem, 0);
						break;
					}					// end switch(cmd)
					break;

				case 0:
					printf("\n\rVoice Memory option not installed");
					break;

// unknown
				default:
					printf("?");

				}						// end switch(c)
			}							// end if
	}									// end function

