//
// JUMA-TRX2 serial I/O test command set
//
// Juha Niinikoski, OH2NLT
//
// Serial test commands moved to own module 03.01.2007
//

// PA RF attenuator test 23.04.2007
// EEPROM Dump limits increaced 08.03.2008
// bar graph test 'B' changed to 'b', better PA100 protocol compatibility  15.11.2008
// <stdio.h> added, lcdpbuff[] type changed to char 05.03.2015

#include <stdio.h>
#include "juma-trx2.h"				// board definitions
#include "DataEEPROM.h"				// EEPROM I/O definitions, interface to assembler module
#include "lcd-trx2.h"				// LCD specific definitions

// external references
// external functions

extern void putch(char);						// UART1 I/O
extern int kbhit(void);
extern unsigned char getch(void);
extern unsigned char getche(void);
extern unsigned long getlong(void);
extern void bin2ascii(unsigned long);
extern void putlong( unsigned long);
extern void rs232_debug(void);

extern void InitUSART2(void);					// UART2 / secondary uart
extern void SetUSART2baud(int);	
extern void putch2(char);
extern int kbhit2(void);
extern unsigned char getch2(void);
extern unsigned char getche2(void);
extern void putst2(char *); 

extern void draw_s_meter(int);					// LCD functions
extern void draw_bar(int, int);
extern void lcd_putst(register const char *);
extern void set_chgen(void);
extern void initlcd(void);
extern void clear_lcd(void);
extern void set_cur_lcd(unsigned char);
extern void disp_freq(long);
extern void disp_rit(int);
extern void disp_meter(int, int);

extern void init_adc12(void);
extern int convert_adc12(unsigned int adchs);	// ADC

extern void init_spi1(void);					// SPI, main board 74HC595 I/O latch
extern void exchg_data_spi1(unsigned int);

extern void start_scaf_clock(void);				// start / stop scaf clocking
extern void stop_scaf_clock(void);

extern void ms_delay( unsigned int);			// general delay


// external data

extern const char ver[];			// system version
extern char lcdpbuff[40];			// LCD print buffer, longer than the display for for prints

extern int batt;					// Powersupply voltage scaled value, measured only when displayed
extern int service_mode;			// service mode selected flag

extern unsigned int long_push;		// long push timer for buttons

extern int s_meter;
extern int cw_pot;

extern unsigned int rf_attn;		// PA RF attenuator
extern int	rf_filt_idx;			// RF filter & attenuator switch table index
extern unsigned int hc595;			// main & PA board SPI bus bits

extern int vfo_idx;					// VFO index, A=0, B=1, if(Split) then  RX=0, TX=1

extern int dds_test;				// DDS test, tuning word +/-
extern int dds_test_idx;			// select tuning word +/- step
extern const long f_test_steps[];
extern int freq_disp_test;			// frequency display, 0 = normal, others = HEX DDS set value 

extern int cw_active;				// status flag for keyer module
extern int tune_on;					// tune state indicator
extern int PTT_IN;					// atomic copy of PTT_IN_IO input / TX on, 0 = active
extern int mute;					// mute flag

// EEPROM structures

#include "trx2_eeprom.h"			// get EEPROM structure definitions

extern union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval)/2];
	}eeprom;

// system calibration values

extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval)/2];
	}cal;


void dump_eeprom(void)
	{
	int x;
	unsigned int y, w;

	printf("Dump EEPROM contents\n\r");	
	for(x=0; x<512; x=x+2)								// byte address for EEPROM, but data is stored in words
		{
		y = ReadEE(EEPAGE, (x+EEDEF), &w, WORD);		// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
		if((x%16) == 0)
			printf("\n\r%5.4X", x);

			printf(" %4X", w);			
		}
	printf("\n\rLast success code = %X ", y);
	}

void serial_test(void)
	{
	int x, z, temp;
	unsigned int y, w;
	unsigned char c;

		if(kbhit())
			{
			c = getch();
			switch(c)
				{

// Info commands
				case 'I':												// system info
					printf("\n\rJUMA-TRX2 copyright OH7SV & OH2NLT\n\r");
					printf("\n\rSoftware version ");
					printf(ver);
					printf(", copyright Juha Niinikoski OH2NLT\n\r");
					break;

// Test commands

// ADC & PWM DAC test
				case 'A':												// Measure all ADC channels
				for(x=9; x<16; x++)
						{
						printf("\n\rADC %i = %i", x, convert_adc12(x));
						}
					break;

// EEPROM
				case 'E':												// dump EEPROM
					dump_eeprom();
					break;

				case 'C':												// clear factory default reset counter
					printf("EEPROM reset counter cleared\n\r");
					w = 0;
					EraseEE(EEPAGE, EE_FD_LOC, WORD);
					WriteEE(&w, EEPAGE, EE_FD_LOC, WORD);
					dump_eeprom();
					break;

// LCD tests
				case 'W':												// test LCD chgen
					printf("LCD write\n\r");
					while(c != 0x1B)
						{
						while(kbhit() == 0);
						c = getch();
					printf("char = %x\n\r", c);
						lcd_putch(c);
						}
					break;

				case 'b':												// bar graph
					printf("LCD bar graph & character test\n\r");
					c = 0;
					clear_lcd();
					for(;;)
					{
					for(x=0; x<96; x++)
						{
						set_cur_lcd(LINE1);
						draw_s_meter(x);
						ms_delay(50);
						}

					set_cur_lcd(LINE2);
					sprintf(lcdpbuff, "%c = 0x%X  ",c, c);		// display LCD characters
					lcd_putst(lcdpbuff);
					c++;

					if(kbhit())
						break;
					}
					break;

// Audio tests

				case 's':												// stop scaf, mute
					printf("\n\rMute");
					mute = 1;											// set flag foe squelch code
					break;

				case 'S':												// start scaf, audio on
					printf("\n\rAudio on");
					mute = 0;
					break;

				case 'p':												// squelch variables
					printf("\n\rS_meter = %1,POT = %i\n\r",s_meter, cw_pot );
					break;


// Temporary hardware debug commands

				case 'd':						// test, debug print some tuning variables
					printf("\n\r eeprom.defval.vfo_sel = %i", eeprom.defval.vfo_sel);
					printf("\n\r vfo_idx = %i", vfo_idx);
					printf("\n\r eeprom.defval.mode[vfo_idx] = %i", eeprom.defval.mode[vfo_idx]);
					printf("\n\r eeprom.defval.mode[2] = %i", eeprom.defval.mode[2]);
					printf("\n\r cw_active = %i", cw_active);
					printf("\n\r tune_on = %i", tune_on);
					printf("\n\r PTT_IN = %i", PTT_IN);
					break;

				case '!':						// special debug
					printf("\n\rSet: eeprom.defval.mode[2] = ");
					c = getch();
					putch(c);
					eeprom.defval.mode[2] = c - '0';
				break;


				case 'D':						// Debug RS232 input, cross copy
					clear_lcd();
					c = 0;

//					printf("\n\r U1STA = %X,  U2STA= %X", U1STA, U2STA);
//					putst2("\n\rJUMA-TRX2 UART2 test\n\r");

					while(c != 0x1B)			// exit with ESC
						{

					if(kbhit())
						{
						c = getch();
						putch2(c);
						set_cur_lcd(LINE1);		
						lcd_putch(c);
						}

					if(kbhit2())
						{
						c = getch2();
						putch(c);
						set_cur_lcd(LINE2);
						lcd_putch(c);
						}

						}
//					rs232_debug();
					break;

				case 'o':						// test, print ref osc calibration value
					printf("\n\rRef Osc = %li MHz\n\r", cal.calval.ref_osc);
					break;


				case 'm':						// test, print ms timing
					printf("\n\rms counter = %i\n\r", get_ms_counter());
					break;

				case 't':						// test write SPI bus
					printf("\n\rSPI Tx");
					exchg_data_spi1(0x5501);
					break;

				case 'c':						// continous test write SPI bus
					printf("\n\rSPI Tx cont");
					for(;;)
						{
						y++;
						exchg_data_spi1(y);
						ms_delay(10);
						if(kbhit())
							break;
						}
					break;

				case '+':						// test
					cal.calval.rf_atn_bits[rf_filt_idx]++;
					cal.calval.rf_atn_bits[rf_filt_idx] = cal.calval.rf_atn_bits[rf_filt_idx] & 0x03;
					printf("\n\rPA Attenuator = 0x%X", cal.calval.rf_atn_bits[rf_filt_idx] );
					break;

				case '-':						// test
					cal.calval.rf_atn_bits[rf_filt_idx]--;
					cal.calval.rf_atn_bits[rf_filt_idx] = cal.calval.rf_atn_bits[rf_filt_idx] & 0x03;
					printf("\n\rPA Attenuator = 0x%X", cal.calval.rf_atn_bits[rf_filt_idx] );	
					break;

				case 'f':						// test, print current SPI bus control bits
					printf("\n\rRF Attn select idx = %i, HC595 bits = 0x%X", rf_filt_idx, hc595);
					break;


				case '0':						// toggle DDS test mode
					if(dds_test == 0)
						{
						printf("\n\rDDS tune test ON");
						dds_test = 1;
						}
					else
						{
						printf("\n\rDDS tune test Off");
						dds_test = 0;
						}
					break;

				case '9':						// toggle DDS freq display mode
					if(freq_disp_test == 0)
						{
						printf("\n\rDDS tuning word display");
						freq_disp_test = 1;
						}
					else
						{
						printf("\n\rNormal Frequency display");
						freq_disp_test = 0;
						}
					break;

				case '1': case '2': case '3': case '4': case '5': case '6':
					dds_test_idx = (c & 0x0F) - 1;
					printf("\n\rDDS tuning step # %i = %li", dds_test_idx, f_test_steps[dds_test_idx]);
					break;


				case 'Z':						// Trap / Error test
					printf("\n\rDiv by Zero");
					x = 0;
					temp = temp / x;			// generate error ==> trap
					break;

				default:
					printf("Key is: %d\n\r", c);
					printf("Try: Valid commands\n\r");

				}	// case
			}		// if
	}
